'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.clone = clone;
exports.extract = extract;

var _child_process = require('child_process');

var _Observable = require('rxjs/Observable');

var _mergeMap = require('rxjs/operator/mergeMap');

var _retryWhen = require('rxjs/operator/retryWhen');

var _cache = require('./cache');

var _util = require('./util');

var util = _interopRequireWildcard(_util);

var _config = require('./config');

var config = _interopRequireWildcard(_config);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _debuglog = require('./debuglog');

var _debuglog2 = _interopRequireDefault(_debuglog);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var log = (0, _debuglog2.default)('git');

function prefixGitArgs() {
	return process.platform === 'win32' ? ['-c', 'core.longpaths=true'] : [];
}

function spawnGit(args) {
	log('executing git with args ' + args);
	var fullArgs = prefixGitArgs().concat(args || []);
	return (0, _child_process.spawn)('git', fullArgs);
}

/**
 * clone a git repository.
 * @param {String} repo - git repository to clone.
 * @param {String} ref - git reference to checkout.
 * @return {Observable} - observable sequence that will be completed once
 * the git repository has been cloned.
 */
function clone(repo, ref) {
	return _Observable.Observable.create(function (observer) {
		var tmpDest = (0, _cache.getTmp)();

		var completeHandler = function completeHandler() {
			observer.next(tmpDest);
			observer.complete();
		};
		var errorHandler = function errorHandler(err) {
			log('failed to clone repository from ' + repo);
			observer.error(err);
		};
		var args = ['clone', '-b', ref, repo, tmpDest, '--single-branch'];
		log('cloning git repository from ' + repo);
		var git = spawnGit(args);
		git.on('close', function (code) {
			return code ? errorHandler() : completeHandler();
		});
	});
}

/**
 * extract a cloned git repository to destination.
 * @param {String} dest - pathname into which the cloned repository should be
 * extracted.
 * @return {Observable} - observable sequence that will be completed once
 * the cloned repository has been extracted.
 */
function extract(dest) {
	var _context;

	var pkgJson = this.pkgJson;
	var target = this.target;

	var tmpDest = pkgJson.dist.path;
	var where = _path2.default.join(dest, '..', config.storageDir, target, 'package');
	return (_context = util.rename(tmpDest, where), _retryWhen.retryWhen).call(_context, function (errors) {
		return _mergeMap.mergeMap.call(errors, function (error) {
			if (error.code !== 'ENOENT') {
				throw error;
			}
			return util.mkdirp(_path2.default.dirname(dest));
		});
	});
}