'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.mergeDependencies = mergeDependencies;
exports.parseBundleDependencies = parseBundleDependencies;
exports.parseDependencies = parseDependencies;
exports.normalizeBin = normalizeBin;
exports.fromFs = fromFs;
exports.updatePkgJson = updatePkgJson;
exports.save = save;
exports.fromArgv = fromArgv;
exports.parseArgv = parseArgv;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ScalarObservable = require('rxjs/observable/ScalarObservable');

var _lodash = require('lodash.frompairs');

var _lodash2 = _interopRequireDefault(_lodash);

var _map = require('rxjs/operator/map');

var _mergeMap = require('rxjs/operator/mergeMap');

var _catch2 = require('rxjs/operator/catch');

var _util = require('./util');

var util = _interopRequireWildcard(_util);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * merge dependency fields.
 * @param  {Object} pkgJson - `package.json` object from which the dependencies
 * should be obtained.
 * @param  {Array.<String>} fields - property names of dependencies to be merged
 * together.
 * @return {Object} - merged dependencies.
 */
function mergeDependencies(pkgJson, fields) {
	var allDependencies = {};
	for (var i = 0; i < fields.length; i++) {
		var field = fields[i];
		var dependencies = pkgJson[field] || {};
		var names = Object.keys(dependencies);
		for (var j = 0; j < names.length; j++) {
			var name = names[j];
			allDependencies[name] = dependencies[name];
		}
	}
	return allDependencies;
}

/**
 * extract an array of bundled dependency names from the passed in
 * `package.json`. uses the `bundleDependencies` and `bundledDependencies`
 * properties.
 * @param  {Object} pkgJson - plain JavaScript object representing a
 * `package.json` file.
 * @return {Array.<String>} - array of bundled dependency names.
 */
function parseBundleDependencies(pkgJson) {
	var bundleDependencies = [].concat(pkgJson.bundleDependencies || []).concat(pkgJson.bundledDependencies || []);
	return bundleDependencies;
}

/**
 * extract specified dependencies from a specific `package.json`.
 * @param  {Object} pkgJson - plain JavaScript object representing a
 * `package.json` file.
 * @param  {Array.<String>} fields - array of dependency fields to be followed.
 * @return {Array} - array of dependency pairs.
 */
function parseDependencies(pkgJson, fields) {
	// bundleDependencies and bundledDependencies are optional. we need to
	// exclude those form the final [name, version] pairs that we're
	// generating.
	var bundleDependencies = parseBundleDependencies(pkgJson);
	var allDependencies = mergeDependencies(pkgJson, fields);
	var names = Object.keys(allDependencies);
	var results = [];
	for (var i = 0; i < names.length; i++) {
		var name = names[i];
		if (bundleDependencies.indexOf(name) === -1) {
			results.push([name, allDependencies[name]]);
		}
	}
	return results;
}

/**
 * normalize the `bin` property in `package.json`, which could either be a
 * string, object or undefined.
 * @param  {Object} pkgJson - plain JavaScript object representing a
 * `package.json` file.
 * @return {Object} - normalized `bin` property.
 */
function normalizeBin(pkgJson) {
	switch (_typeof(pkgJson.bin)) {
		case 'string':
			return _defineProperty({}, pkgJson.name, pkgJson.bin);
		case 'object':
			return pkgJson.bin;
		default:
			return {};
	}
}

/**
 * create an instance by reading a `package.json` from disk.
 * @param  {String} baseDir - base directory of the project.
 * @return {Observabel} - an observable sequence of an `EntryDep`.
 */
function fromFs(baseDir) {
	var filename = _path2.default.join(baseDir, 'package.json');
	return util.readFileJSON(filename);
}

function updatePkgJson(pkgJson, diff) {
	var updatedPkgJson = _extends({}, pkgJson);
	var fields = Object.keys(diff);
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = fields[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var field = _step.value;

			updatedPkgJson[field] = _extends({}, updatedPkgJson[field], diff[field]);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return updatedPkgJson;
}

function save(baseDir) {
	var _context;

	var filename = _path2.default.join(baseDir, 'package.json');

	return (_context = (_context = _mergeMap.mergeMap.call(this, function (diff) {
		var _context2;

		return (_context2 = (_context2 = fromFs(baseDir), _catch2._catch).call(_context2, function () {
			return _ScalarObservable.ScalarObservable.create({});
		}), _map.map).call(_context2, function (pkgJson) {
			return updatePkgJson(pkgJson, diff);
		});
	}), _map.map).call(_context, function (pkgJson) {
		return JSON.stringify(pkgJson, null, '\t');
	}), _mergeMap.mergeMap).call(_context, function (pkgJson) {
		return util.writeFile(filename, pkgJson, 'utf8');
	});
}

/**
 * create an instance by parsing the explicit dependencies supplied via
 * command line arguments.
 * @param  {String} baseDir - base directory of the project.
 * @param  {Array} argv - command line arguments.
 * @return {Observabel} - an observable sequence of an `EntryDep`.
 */
function fromArgv(baseDir, argv) {
	var pkgJson = parseArgv(argv);
	return _ScalarObservable.ScalarObservable.create(pkgJson);
}

/**
 * parse the command line arguments and create the dependency field of a
 * `package.json` file from it.
 * @param  {Array} argv - command line arguments.
 * @return {NullPkgJSON} - package.json created from explicit dependencies
 * supplied via command line arguments.
 */
function parseArgv(argv) {
	var names = argv._.slice(1);

	var nameVersionPairs = (0, _lodash2.default)(names.map(function (target) {
		var nameVersion = /^(@?.+?)(?:@(.+)?)?$/.exec(target);
		return [nameVersion[1], nameVersion[2] || '*'];
	}));

	var field = argv['save-dev'] ? 'devDependencies' : argv['save-optional'] ? 'optionalDependencies' : 'dependencies';

	return _defineProperty({}, field, nameVersionPairs);
}